# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import dataiku
from openai import OpenAI
import json

MODEL = "gpt-3.5-turbo-0613"

df = dataiku.Dataset("product_reviews").get_dataframe()

auth_info = dataiku.api_client().get_auth_info(with_secrets=True)
for secret in auth_info["secrets"]:
    if secret["key"] == "openai_key":
        client = OpenAI(api_key=secret["value"])
        break

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
functions = [
    {
        "name": "extract_data",
        "description": "Save key information from product reviews",
        "parameters": {
            "type": "object",
                "properties": {
                    "product": {
                        "type": "string",
                        "description": "type of the product ('unknown' if not provided; if the model or brand is included, be as specific as possible)"
                    },
                    "summary": {
                        "type": "string",
                        "description": "summary of the product review in less than 10 words"
                    },
                    "predicted_sentiment": {
                        "type": "string",
                        "enum": ["positive", "neutral", "negative"],
                        "description": "sentiment of the product review"
                    },
                    "advantages": {
                        "type": "array",
                        "items": {
                            "type": "string"
                        },
                        "description": "main advantages mentioned in the product review"
                    },
                    "drawbacks": {
                        "type": "array",
                        "items": {
                            "type": "string"
                        },
                        "description": "main drawbacks mentioned in the product review"
                    }
                },
            "required": ["product", "summary", "predicted_sentiment", "advantages", "drawbacks"],
        },
    }
]

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
for i in df.index:
    messages = [
        {
            "role": "system",
            "content": "You are a helpful assistant that extracts key information from product reviews."
        },
        {
            "role": "user",
            "content": 'Extract key information from the following product review: ' + df.loc[i, "text"]
        }
    ]
    response = client.chat.completions.create(
        model=MODEL,
        functions=functions,
        function_call={"name": "extract_data"},
        messages=messages
    )
    df.loc[i, "json"] = response.choices[0].message.function_call.arguments
    try:
        d = json.loads(df.loc[i, "json"])
        for k in d:
            df.loc[i, k] = json.dumps(d[k]) if k in ["advantages", "drawbacks"] else d[k]
    except json.JSONDecodeError:
        pass

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
dataiku.Dataset("extracted_json_openai_function_call").write_with_schema(df)