# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import dataiku
import numpy as np
from openai import OpenAI
import tiktoken

MODEL = "gpt-3.5-turbo"

enc = tiktoken.encoding_for_model(MODEL)

df = dataiku.Dataset("product_reviews").get_dataframe()

# Provide the OpenAI API key
auth_info = dataiku.api_client().get_auth_info(with_secrets=True)
for secret in auth_info["secrets"]:
    if secret["key"] == "openai_key":
        client = OpenAI(api_key=secret["value"])
        break

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
instructions = "Decide whether a product review's sentiment is positive, neutral, or negative"

# We add few-shot examples to the prompt
examples = [
    (
        "I love my new chess board!",
        "positive"
    ),
    (
        "Not what I expected but I guess it'll do",
        "neutral"
    ),
    (
        "I'm so disappointed. The product seemed much better on the website",
        "negative"
    )
]
labels = set()
prompt = [{"role": "system", "content": instructions}]
for text, sentiment in examples:
    prompt += [
        {"role": "system", "name": "example_user", "content": text},
        {"role": "system", "name": "example_assistant", "content": sentiment}
    ]
    labels.add(sentiment)

# We adjust the logit biases so that only the allowed tokens can be generated
# The code below works only when the class labels are made of only 1 token
max_tokens, logit_bias = 0, {}
for label in labels:
    tokens = enc.encode(label)
    max_tokens = max(max_tokens, len(tokens))
    for token in tokens:
        logit_bias[str(token)] = 100

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
label2column = {label: f"proba_{label}" for label in labels}

for i in df.index:
    # Call the Chat Completion endpoint of the OpenAI API
    result = client.chat.completions.create(
        model=MODEL,
        messages=(
            prompt
            + [{"role": "user", "content": df.at[i, "text"]}]
        ),
        logprobs=True,
        max_tokens=max_tokens,
        logit_bias=logit_bias
    )

    df.at[i, "prediction"] = result.choices[0].logprobs.content[0].token
    df.at[i, "score"] = np.exp(result.choices[0].logprobs.content[0].logprob)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
dataiku.Dataset("sentiment_openai_chat").write_with_schema(df)