# Specifications of the py_39_sample_llm_mistral code environment

To create this code environment, you will need a DSS instance hosted on a VM with at least 64 GB RAM.

## Packages

You need to specify extra options for "pip install" (cf. "General" tab of the code environment): `--index-url https://download.pytorch.org/whl/cu117 --extra-index-url https://pypi.org/simple` (if you use CUDA 11.7).

![extra_options.png](b94XOupTu3MJ)

```
accelerate==0.25.0
bitsandbytes==0.41.3.post2
datasets==2.15.0
peft @ git+https://github.com/huggingface/peft.git@21c304f6f6ea62c7dcbef8e201d178a7575b471d
scipy==1.11.4
torch==2.0.1
transformers @ git+https://github.com/huggingface/transformers.git@680c610f9733561fb09e21c0e6ed513d69564f56
trl @ git+https://github.com/huggingface/trl.git@c0ce52ab26e71810b11996
```

## Resource initialization script

These resources may not be needed if the corresponding recipes are not used (cf. comments in the script).

```
## Base imports
import os

from dataiku.code_env_resources import clear_all_env_vars
from dataiku.code_env_resources import grant_permissions
from dataiku.code_env_resources import set_env_path
from dataiku.code_env_resources import set_env_var

# Clears all environment variables defined by previously run script
clear_all_env_vars()

## Hugging Face
# Set HuggingFace cache directory
set_env_path("HF_HOME", "huggingface")
set_env_path("TRANSFORMERS_CACHE", "huggingface/transformers")
hf_home_dir = os.getenv("HF_HOME")
transformers_home_dir = os.getenv("TRANSFORMERS_CACHE")

# Import Hugging Face's transformers and PyTorch
import transformers
from transformers import BitsAndBytesConfig

# Download pre-trained model
MODEL_NAME = "mistralai/Mistral-7B-Instruct-v0.2"
MODEL_REVISION = "c72e5d1908b1e2929ec8fc4c8820e9706af1f80f"

model = transformers.AutoModelForCausalLM.from_pretrained(
    MODEL_NAME,
    revision=MODEL_REVISION
)
tokenizer = transformers.AutoTokenizer.from_pretrained(
    MODEL_NAME,
    revision=MODEL_REVISION
)

#from accelerate.utils import write_basic_config
#write_basic_config(mixed_precision='fp16')

# Grant everyone read access to pre-trained models in the HF_HOME folder
# (by default, only readable by the owner)
grant_permissions(hf_home_dir)
grant_permissions(transformers_home_dir)
```